

//Language tool versions: mplink.exe v4.40, mcc18.exe v3.40



/********************************************************************
 FileName:		Keyboard.c
 Dependencies: See INCLUDES section
 Processor:		PIC18, PIC24, and PIC32 USB Microcontrollers
 Hardware:		This demo is natively intended to be used on Microchip USB demo
 				boards supported by the MCHPFSUSB stack.  See release notes for
 				support matrix.  This demo can be modified for use on other hardware
 				platforms.
 Complier:  	Microchip C18 (for PIC18), C30 (for PIC24), C32 (for PIC32)
 Company:		Microchip Technology, Inc.

 Software License Agreement:

********************************************************************/
//---------------------------------------------------------------------
//---------------------------------------------------------------------
#ifndef KEYBOARD_C
#define KEYBOARD_C

/** INCLUDES *******************************************************/
#include "./USB/usb.h"
#include "HardwareProfile.h"
#include "./USB/usb_function_hid.h"
#include "i2c.h"

/** CONFIGURATION **************************************************/
#if defined(PICDEM_FS_USB)      // Configuration bits for PICDEM FS USB Demo Board (based on PIC18F4550)
        #pragma config PLLDIV   = 5         // (20 MHz crystal on PICDEM FS USB board)
        #pragma config CPUDIV   = OSC1_PLL2   
        #pragma config USBDIV   = 2         // Clock source from 96MHz PLL/2
        #pragma config FOSC     = HSPLL_HS
        #pragma config FCMEN    = OFF
        #pragma config IESO     = OFF
        #pragma config PWRT     = OFF
        #pragma config BOR      = ON
        #pragma config BORV     = 3
        #pragma config VREGEN   = ON      //USB Voltage Regulator
        #pragma config WDT      = OFF
        #pragma config WDTPS    = 32768
        #pragma config MCLRE    = ON
        #pragma config LPT1OSC  = OFF
        #pragma config PBADEN   = OFF
//      #pragma config CCP2MX   = ON
        #pragma config STVREN   = ON
        #pragma config LVP      = OFF
//      #pragma config ICPRT    = OFF       // Dedicated In-Circuit Debug/Programming
        #pragma config XINST    = OFF       // Extended Instruction Set
        #pragma config CP0      = OFF
        #pragma config CP1      = OFF
//      #pragma config CP2      = OFF
//      #pragma config CP3      = OFF
        #pragma config CPB      = OFF
//      #pragma config CPD      = OFF
        #pragma config WRT0     = OFF
        #pragma config WRT1     = OFF
//      #pragma config WRT2     = OFF
//      #pragma config WRT3     = OFF
        #pragma config WRTB     = OFF       // Boot Block Write Protection
        #pragma config WRTC     = OFF
//      #pragma config WRTD     = OFF
        #pragma config EBTR0    = OFF
        #pragma config EBTR1    = OFF
//      #pragma config EBTR2    = OFF
//      #pragma config EBTR3    = OFF
        #pragma config EBTRB    = OFF

#elif defined(LOW_PIN_COUNT_USB_DEVELOPMENT_KIT)
        #pragma config CPUDIV = NOCLKDIV
        #pragma config USBDIV = OFF
        #pragma config FOSC   = HS
        #pragma config PLLEN  = ON
        #pragma config FCMEN  = OFF
        #pragma config IESO   = OFF
        #pragma config PWRTEN = OFF
        #pragma config BOREN  = OFF
        #pragma config BORV   = 30
        #pragma config WDTEN  = OFF
        #pragma config WDTPS  = 32768
        #pragma config MCLRE  = OFF
        #pragma config HFOFST = OFF
        #pragma config STVREN = ON
        #pragma config LVP    = OFF
        #pragma config XINST  = OFF
        #pragma config BBSIZ  = OFF
        #pragma config CP0    = OFF
        #pragma config CP1    = OFF
        #pragma config CPB    = OFF
        #pragma config WRT0   = OFF
        #pragma config WRT1   = OFF
        #pragma config WRTB   = OFF
        #pragma config WRTC   = OFF
        #pragma config EBTR0  = OFF
        #pragma config EBTR1  = OFF
        #pragma config EBTRB  = OFF        

#else
    #error No hardware board defined, see "HardwareProfile.h" and __FILE__
#endif

/** VARIABLES ******************************************************/

BYTE old_sw2,old_sw3;
char buffer[8];
unsigned char OutBuffer[8];
USB_HANDLE lastINTransmission;
USB_HANDLE lastOUTTransmission;
BOOL Keyboard_out;
BOOL BlinkStatusValid;
DWORD CountdownTimerToShowUSBStatusOnLEDs;

BYTE i2c_add;	//I2C slave device address
BYTE mode_ASCII;

/** PRIVATE PROTOTYPES *********************************************/
void BlinkUSBStatus(void);
BOOL Switch2IsPressed(void);
BOOL Switch3IsPressed(void);
static void InitializeSystem(void);
void ProcessIO(void);
void UserInit(void);
void YourHighPriorityISRCode();
void YourLowPriorityISRCode();
void USBCBSendResume(void);
void Keyboard(BYTE cts, BYTE eks);
void Keyboard2(BYTE cts, BYTE eks);
void Keyboard3(unsigned char cts);
void Keyboard5(BYTE v_n1, BYTE v_n2, BYTE v_n3);
void mDelay(int ccnt);
void SendKeyBit(int v_bit);
void SendConfig(void);
void SendString1(void);
void SendString2(void);

void USBHIDCBSetReportComplete(void);


/********************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *******************************************************************/
#if defined(__18CXX)
void main(void)
#else
int main(void)
#endif
{
    InitializeSystem();

    #if defined(USB_INTERRUPT)
        USBDeviceAttach();
    #endif

    while(1)
    {
        #if defined(USB_POLLING)
		// Check bus status and service USB interrupts.
        USBDeviceTasks(); // Interrupt or polling method.  If using polling, must call
        				  // this function periodically.  This function will take care
        				  // of processing and responding to SETUP transactions 
        				  // (such as during the enumeration process when you first
        				  // plug in).  USB hosts require that USB devices should accept
        				  // and process SETUP packets in a timely fashion.  Therefore,
        				  // when using polling, this function should be called 
        				  // regularly (such as once every 1.8ms or faster** [see 
        				  // inline code comments in usb_device.c for explanation when
        				  // "or faster" applies])  In most cases, the USBDeviceTasks() 
        				  // function does not take very long to execute (ex: <100 
        				  // instruction cycles) before it returns.
        #endif
    				  

		// Application-specific tasks.
		// Application related code may be added here, or in the ProcessIO() function.
        ProcessIO();        
    }//end while
}//end main


/********************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization
 *                  routine. All required USB initialization routines
 *                  are called from here.
 *
 *                  User application initialization routine should
 *                  also be called from here.                  
 *
 * Note:            None
 *******************************************************************/
static void InitializeSystem(void)
{
    #if (defined(__18CXX) & !defined(PIC18F87J50_PIM))
        ADCON1 |= 0x0F;                 // Default all pins to digital
    #elif defined(__C30__)
        #if defined(__PIC24FJ256GB110__) || defined(__PIC24FJ256GB106__)
            AD1PCFGL = 0xFFFF;
         #elif defined(__dsPIC33EP512MU810__)||defined(__PIC24EP512GU810__)
        	ANSELA = 0x0000;
    		ANSELB = 0x0000;
    		ANSELC = 0x0000;
    		ANSELD = 0x0000;
    		ANSELE = 0x0000;
    		ANSELG = 0x0000;
            
            // The dsPIC33EP512MU810 features Peripheral Pin
            // select. The following statements map UART2 to 
            // device pins which would connect to the the 
            // RX232 transciever on the Explorer 16 board.

             RPINR19 = 0;
             RPINR19 = 0x64;
             RPOR9bits.RP101R = 0x3;

        #endif
    #elif defined(__C32__)
        AD1PCFG = 0xFFFF;
    #endif

    #if defined(PIC18F87J50_PIM) || defined(PIC18F46J50_PIM) || defined(PIC18F_STARTER_KIT_1) || defined(PIC18F47J53_PIM)
	//On the PIC18F87J50 Family of USB microcontrollers, the PLL will not power up and be enabled
	//by default, even if a PLL enabled oscillator configuration is selected (such as HS+PLL).
	//This allows the device to power up at a lower initial operating frequency, which can be
	//advantageous when powered from a source which is not gauranteed to be adequate for 48MHz
	//operation.  On these devices, user firmware needs to manually set the OSCTUNE<PLLEN> bit to
	//power up the PLL.
    {
        unsigned int pll_startup_counter = 600;
        OSCTUNEbits.PLLEN = 1;  //Enable the PLL and wait 2+ms until the PLL locks before enabling USB module
        while(pll_startup_counter--);
    }
    //Device switches over automatically to PLL output after PLL is locked and ready.
    #endif

    #if defined(PIC18F87J50_PIM)
	//Configure all I/O pins to use digital input buffers.  The PIC18F87J50 Family devices
	//use the ANCONx registers to control this, which is different from other devices which
	//use the ADCON1 register for this purpose.
    WDTCONbits.ADSHR = 1;			// Select alternate SFR location to access ANCONx registers
    ANCON0 = 0xFF;                  // Default all pins to digital
    ANCON1 = 0xFF;                  // Default all pins to digital
    WDTCONbits.ADSHR = 0;			// Select normal SFR locations
    #endif

    #if defined(PIC18F46J50_PIM) || defined(PIC18F_STARTER_KIT_1) || defined(PIC18F47J53_PIM)
	//Configure all I/O pins to use digital input buffers.  The PIC18F87J50 Family devices
	//use the ANCONx registers to control this, which is different from other devices which
	//use the ADCON1 register for this purpose.
    ANCON0 = 0xFF;                  // Default all pins to digital
    ANCON1 = 0xFF;                  // Default all pins to digital
    #endif
    
   #if defined(PIC24FJ64GB004_PIM) || defined(PIC24FJ256DA210_DEV_BOARD)
	//On the PIC24FJ64GB004 Family of USB microcontrollers, the PLL will not power up and be enabled
	//by default, even if a PLL enabled oscillator configuration is selected (such as HS+PLL).
	//This allows the device to power up at a lower initial operating frequency, which can be
	//advantageous when powered from a source which is not gauranteed to be adequate for 32MHz
	//operation.  On these devices, user firmware needs to manually set the CLKDIV<PLLEN> bit to
	//power up the PLL.
    {
        unsigned int pll_startup_counter = 600;
        CLKDIVbits.PLLEN = 1;
        while(pll_startup_counter--);
    }

    //Device switches over automatically to PLL output after PLL is locked and ready.
    #endif
    
    #if defined(__32MX460F512L__)|| defined(__32MX795F512L__)
    // Configure the PIC32 core for the best performance
    // at the operating frequency. The operating frequency is already set to 
    // 60MHz through Device Config Registers
    SYSTEMConfigPerformance(60000000);
	#endif

#if defined(__dsPIC33EP512MU810__)||defined(__PIC24EP512GU810__)

    // Configure the device PLL to obtain 60 MIPS operation. The crystal
    // frequency is 8MHz. Divide 8MHz by 2, multiply by 60 and divide by
    // 2. This results in Fosc of 120MHz. The CPU clock frequency is
    // Fcy = Fosc/2 = 60MHz. Wait for the Primary PLL to lock and then
    // configure the auxilliary PLL to provide 48MHz needed for USB 
    // Operation.

	PLLFBD = 58;				/* M  = 60	*/
	CLKDIVbits.PLLPOST = 0;		/* N1 = 2	*/
	CLKDIVbits.PLLPRE = 0;		/* N2 = 2	*/
	OSCTUN = 0;			

    /*	Initiate Clock Switch to Primary
     *	Oscillator with PLL (NOSC= 0x3)*/
	
    __builtin_write_OSCCONH(0x03);		
	__builtin_write_OSCCONL(0x01);
	
	while (OSCCONbits.COSC != 0x3);       

    // Configuring the auxiliary PLL, since the primary
    // oscillator provides the source clock to the auxiliary
    // PLL, the auxiliary oscillator is disabled. Note that
    // the AUX PLL is enabled. The input 8MHz clock is divided
    // by 2, multiplied by 24 and then divided by 2. Wait till 
    // the AUX PLL locks.

    ACLKCON3 = 0x24C1;   
    ACLKDIV3 = 0x7;
    
    ACLKCON3bits.ENAPLL = 1;
    while(ACLKCON3bits.APLLCK != 1); 

    #endif


//	The USB specifications require that USB peripheral devices must never source
//	current onto the Vbus pin.  Additionally, USB peripherals should not source
//	current on D+ or D- when the host/hub is not actively powering the Vbus line.
//	When designing a self powered (as opposed to bus powered) USB peripheral
//	device, the firmware should make sure not to turn on the USB module and D+
//	or D- pull up resistor unless Vbus is actively powered.  Therefore, the
//	firmware needs some means to detect when Vbus is being powered by the host.
//	A 5V tolerant I/O pin can be connected to Vbus (through a resistor), and
// 	can be used to detect when Vbus is high (host actively powering), or low
//	(host is shut down or otherwise not supplying power).  The USB firmware
// 	can then periodically poll this I/O pin to know when it is okay to turn on
//	the USB module/D+/D- pull up resistor.  When designing a purely bus powered
//	peripheral device, it is not possible to source current on D+ or D- when the
//	host is not actively providing power on Vbus. Therefore, implementing this
//	bus sense feature is optional.  This firmware can be made to use this bus
//	sense feature by making sure "USE_USB_BUS_SENSE_IO" has been defined in the
//	HardwareProfile.h file.    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See HardwareProfile.h
    #endif
    
//	If the host PC sends a GetStatus (device) request, the firmware must respond
//	and let the host know if the USB peripheral device is currently bus powered
//	or self powered.  See chapter 9 in the official USB specifications for details
//	regarding this request.  If the peripheral device is capable of being both
//	self and bus powered, it should not return a hard coded value for this request.
//	Instead, firmware should check if it is currently self or bus powered, and
//	respond accordingly.  If the hardware has been configured like demonstrated
//	on the PICDEM FS USB Demo Board, an I/O pin can be polled to determine the
//	currently selected power source.  On the PICDEM FS USB Demo Board, "RA2" 
//	is used for	this purpose.  If using this feature, make sure "USE_SELF_POWER_SENSE_IO"
//	has been defined in HardwareProfile.h, and that an appropriate I/O pin has been mapped
//	to it in HardwareProfile.h.
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;	// See HardwareProfile.h
    #endif
    
    UserInit();

    USBDeviceInit();	//usb_device.c.  Initializes USB module SFRs and firmware
    					//variables to known states.
}//end InitializeSystem


//---------------------------------------------------------------------
//---------------------------------------------------------------------
void UserInit(void)
{
    //Initialize all of the LED pins
    mInitAllLEDs();
    BlinkStatusValid = TRUE;
    
    //Initialize all of the push buttons
    mInitAllSwitches();
    old_sw2 = sw2;
    old_sw3 = sw3;

    lastINTransmission = 0;
    lastOUTTransmission = 0;

    TRISCbits.TRISC2 = 0;	//USB LED

	TRISBbits.TRISB7 = 1;
	TRISBbits.TRISB6 = 1;
	TRISBbits.TRISB5 = 1;
	TRISBbits.TRISB4 = 1;
	INTCON2bits.RABPU = 0;	//enable Pull-Ups

    ANSEL = 0x0000;
    ANSELH = 0x0000;
	WPUB = 0b11110000;

	mode_ASCII = 1;			//no config allowed in this version

}//end UserInit

//---------------------------------------------------------------------
//---------------------------------------------------------------------
void ProcessIO(void)
{   	
    static int sww = 0;
	int ix;

    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

	//check if switch on the board for status send has been pressed
    if(PORTBbits.RB7 == 0 && sww == 0)
       	{
            mLED_3_On();
			SendString1();
			sww = 1;
       	}
    else
       	{
    		if(PORTBbits.RB7 == 1 && PORTBbits.RB6 == 1)
			{
				sww = 0;
			}
            mLED_3_Off();
  		}

    if(PORTBbits.RB6 == 0 && sww == 0)
       	{
            mLED_3_On();
			SendString2();
			sww = 1;
       	}
    else
       	{
    		if(PORTBbits.RB7 == 1 && PORTBbits.RB6 == 1)
			{
				sww = 0;
			}
            mLED_3_Off();
  		}
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void SendString2(void)
{
	//Mariana_1971san + ENTER
	mLED_3_On();
	Keyboard3('T');
	Keyboard3('o');
	Keyboard3('t');
	Keyboard3('o');
	Keyboard3('k');
	Keyboard3(' ');
	Keyboard3('j');
	Keyboard3('e');
	Keyboard3(' ');
	Keyboard3('h');
	Keyboard3('e');
	Keyboard3('s');
	Keyboard3('l');
	Keyboard3('o');
	Keyboard3('1');
	Keyboard2(40,0);
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void SendString1(void)
{
	//Mariana_1971san w/o ENTER
	mLED_3_On();
	Keyboard3('T');
	Keyboard3('o');
	Keyboard3('t');
	Keyboard3('o');
	Keyboard3('k');
	Keyboard3(' ');
	Keyboard3('j');
	Keyboard3('e');
	Keyboard3(' ');
	Keyboard3('h');
	Keyboard3('e');
	Keyboard3('s');
	Keyboard3('l');
	Keyboard3('o');
	Keyboard3('2');
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void SendConfig(void)
{
			UserInit();		//read the actual values from sw and perform I2C initialization

			mLED_3_On();

			Keyboard3('I');
			Keyboard3('2');
			Keyboard3('C');
			Keyboard3(' ');
			Keyboard3('t');
			Keyboard3('o');
			Keyboard3(' ');
			Keyboard3('U');
			Keyboard3('S');
			Keyboard3('B');
			Keyboard3(' ');
			Keyboard3('B');
			Keyboard3('r');
			Keyboard3('i');
			Keyboard3('d');
			Keyboard3('g');
			Keyboard3('e');
			Keyboard3(' ');
			Keyboard5(98,92,95);		// character: /
			Keyboard3(' ');
			Keyboard3('F');
			Keyboard3('i');
			Keyboard3('r');
			Keyboard3('m');
			Keyboard3('w');
			Keyboard3('a');
			Keyboard3('r');
			Keyboard3('e');
			Keyboard3(' ');
			Keyboard3('v');
			Keyboard3('1');
			Keyboard3('.');
			Keyboard3('0');
			Keyboard3('8');
			Keyboard3(' ');
			Keyboard5(98,92,95);		// character: /
			Keyboard3(' ');
			Keyboard3('1');
			Keyboard3('2');
			Keyboard3('.');
			Keyboard3('0');
			Keyboard3('5');
			Keyboard3('.');
			Keyboard3('2');
			Keyboard3('0');
			Keyboard3('2');
			Keyboard3('0');
			Keyboard2(40,0);
			Keyboard3('I');
			Keyboard3('2');
			Keyboard3('C');
			Keyboard3(' ');
			Keyboard3('A');
			Keyboard3('d');
			Keyboard3('d');
			Keyboard3('r');
			Keyboard3('e');
			Keyboard3('s');
			Keyboard3('s');
			Keyboard5(98,93,96);		// character: :
			Keyboard3(' ');
			Keyboard3('0');
			Keyboard3('1');
			Keyboard3('1');
			Keyboard3('0');
			SendKeyBit(PORTCbits.RC6);
			SendKeyBit(PORTCbits.RC5);
			SendKeyBit(PORTCbits.RC4);
			Keyboard2(40,0);

			Keyboard3('S');
			Keyboard3('e');
			Keyboard3('l');
			Keyboard3('e');
			Keyboard3('c');
			Keyboard3('t');
			Keyboard3('e');
			Keyboard3('d');
			Keyboard3(' ');
			Keyboard3('m');
			Keyboard3('o');
			Keyboard3('d');
			Keyboard3('e');
			Keyboard3(' ');
			Keyboard3('i');
			Keyboard3('s');
			Keyboard3(' ');
			if(mode_ASCII == 1)
			{
				Keyboard3('A');
				Keyboard3('S');
				Keyboard3('C');
				Keyboard3('I');
				Keyboard3('I');
				Keyboard3(' ');
				Keyboard3('w');
				Keyboard3('i');
				Keyboard3('t');
				Keyboard3('h');
				Keyboard3(' ');
				Keyboard3('E');
				Keyboard3('N');
				Keyboard3(' ');
				Keyboard3('e');
				Keyboard3('n');
				Keyboard3('c');
				Keyboard3('o');
				Keyboard3('d');
				Keyboard3('i');
				Keyboard3('n');
				Keyboard3('g');
			}
			else
			{
				Keyboard3('d');
				Keyboard3('i');
				Keyboard3('r');
				Keyboard3('e');
				Keyboard3('c');
				Keyboard3('t');
			}
			Keyboard2(40,0);
			Keyboard2(40,0);
			Keyboard2(40,0);
}

//---------------------------------------------------------------------
//---------------------------------------------------------------------
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void SendKeyBit(int v_bit)
{
	if(v_bit == 0)
	{
		Keyboard2(39,0);		//0
	}
	else
	{
		Keyboard2(30,0);		//1
	}
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void mDelay(int ccnt)
{
    int aa;
	aa = ccnt;
	while(aa)
	{
		aa--;
	}
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void Keyboard3(unsigned char cts)
{
// coding for english keyboard is implemented here

	switch(cts)
	{
		case 'a': Keyboard2(4,0); return; 
		case 'b': Keyboard2(5,0); return; 
		case 'c': Keyboard2(6,0); return; 
		case 'd': Keyboard2(7,0); return; 
		case 'e': Keyboard2(8,0); return; 
		case 'f': Keyboard2(9,0); return; 
		case 'g': Keyboard2(10,0); return; 
		case 'h': Keyboard2(11,0); return; 
		case 'i': Keyboard2(12,0); return; 
		case 'j': Keyboard2(13,0); return; 
		case 'k': Keyboard2(14,0); return; 
		case 'l': Keyboard2(15,0); return; 
		case 'm': Keyboard2(16,0); return; 
		case 'n': Keyboard2(17,0); return; 
		case 'o': Keyboard2(18,0); return; 
		case 'p': Keyboard2(19,0); return; 
		case 'q': Keyboard2(20,0); return; 
		case 'r': Keyboard2(21,0); return; 
		case 's': Keyboard2(22,0); return; 
		case 't': Keyboard2(23,0); return; 
		case 'u': Keyboard2(24,0); return; 
		case 'v': Keyboard2(25,0); return; 
		case 'w': Keyboard2(26,0); return; 
		case 'x': Keyboard2(27,0); return; 
		case 'y': Keyboard2(28,0); return; 
		case 'z': Keyboard2(29,0); return; 
		case '1': Keyboard2(30,0); return; 
		case '2': Keyboard2(31,0); return; 
		case '3': Keyboard2(32,0); return; 
		case '4': Keyboard2(33,0); return; 
		case '5': Keyboard2(34,0); return; 
		case '6': Keyboard2(35,0); return; 
		case '7': Keyboard2(36,0); return; 
		case '8': Keyboard2(37,0); return; 
		case '9': Keyboard2(38,0); return; 
		case '0': Keyboard2(39,0); return; 
		case ' ': Keyboard2(44,0); return;
		case '.': Keyboard2(55,0); return;
		case '/': Keyboard2(56,0); return;
		case '-': Keyboard2(45,0); return;
		case '=': Keyboard2(46,0); return;
		case '[': Keyboard2(47,0); return;
		case ']': Keyboard2(48,0); return;
		case '+': Keyboard2(46,2); return;
		case '>': Keyboard2(55,2); return;
		case ':': Keyboard2(51,2); return;
		case 13:  Keyboard2(40,0); return;
		case 'A': Keyboard2(4,2); return; 
		case 'B': Keyboard2(5,2); return; 
		case 'C': Keyboard2(6,2); return; 
		case 'D': Keyboard2(7,2); return; 
		case 'E': Keyboard2(8,2); return; 
		case 'F': Keyboard2(9,2); return; 
		case 'G': Keyboard2(10,2); return; 
		case 'H': Keyboard2(11,2); return; 
		case 'I': Keyboard2(12,2); return; 
		case 'J': Keyboard2(13,2); return; 
		case 'K': Keyboard2(14,2); return; 
		case 'L': Keyboard2(15,2); return; 
		case 'M': Keyboard2(16,2); return; 
		case 'N': Keyboard2(17,2); return; 
		case 'O': Keyboard2(18,2); return; 
		case 'P': Keyboard2(19,2); return; 
		case 'Q': Keyboard2(20,2); return; 
		case 'R': Keyboard2(21,2); return; 
		case 'S': Keyboard2(22,2); return; 
		case 'T': Keyboard2(23,2); return; 
		case 'U': Keyboard2(24,2); return; 
		case 'V': Keyboard2(25,2); return; 
		case 'W': Keyboard2(26,2); return; 
		case 'X': Keyboard2(27,2); return; 
		case 'Y': Keyboard2(28,2); return; 
		case 'Z': Keyboard2(29,2); return;

		case '@': Keyboard2(20,64); return;    //for DE-keyboard only
//		case '@': Keyboard5(98,94,92); return;
	}
	Keyboard2(56,2);		//not recognized character - write just "?"
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void Keyboard5(BYTE v_n1, BYTE v_n2, BYTE v_n3)
{
	static int ddd = 1500;

	Keyboard(98, 4);
	mDelay(ddd);
	USBDeviceTasks();
	Keyboard(v_n1, 4);
	mDelay(ddd);
	USBDeviceTasks();
	Keyboard(v_n2, 4);
	mDelay(ddd);
	USBDeviceTasks();
	Keyboard(v_n3, 4);
	mDelay(ddd);
	USBDeviceTasks();

	Keyboard(0,0);
	mDelay(ddd);
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void Keyboard2(BYTE cts, BYTE eks)
{

// Key codes - see Hut1_12v2.pdf; Page 53

	static int ddd = 1500;

	Keyboard(cts, eks);
	mDelay(ddd);
	USBDeviceTasks();

	Keyboard(0,0);
	mDelay(ddd);
}
//---------------------------------------------------------------------
//---------------------------------------------------------------------
void Keyboard(BYTE cts, BYTE eks)
{
	//Check if the IN endpoint is not busy, and if it isn't check if we want to send 
	//keystroke data to the host.
    if(!HIDTxHandleBusy(lastINTransmission))
    {
        	//Load the HID buffer
        	hid_report_in[0] = eks;
        	hid_report_in[1] = 0;
        	hid_report_in[2] = cts;
        	hid_report_in[3] = 0;
        	hid_report_in[4] = 0;
        	hid_report_in[5] = 0;
        	hid_report_in[6] = 0;
        	hid_report_in[7] = 0;
           	//Send the 8 byte packet over USB to the host.
           	lastINTransmission = HIDTxPacket(HID_EP, (BYTE*)hid_report_in, 0x08);

    }

// hid_report_in[0] / eks:

// bit 0: LEFT CTRL 
// bit 1: LEFT SHIFT 
// bit 2: LEFT ALT 
// bit 3: LEFT GUI 
// bit 4: RIGHT CTRL 
// bit 5: RIGHT SHIFT 
// bit 6: RIGHT ALT 
// bit 7: RIGHT GUI

    return;		
}//end keyboard()

//---------------------------------------------------------------------
//---------------------------------------------------------------------

BOOL Switch2IsPressed(void)
{
    if(sw2 != old_sw2)
    {
        old_sw2 = sw2;                  // Save new value
        if(sw2 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch2IsPressed

//---------------------------------------------------------------------
//---------------------------------------------------------------------
BOOL Switch3IsPressed(void)
{
    if(sw3 != old_sw3)
    {
        old_sw3 = sw3;                  // Save new value
        if(sw3 == 0)                    // If pressed
            return TRUE;                // Was pressed
    }//end if
    return FALSE;                       // Was not pressed
}//end Switch3IsPressed

//---------------------------------------------------------------------
//---------------------------------------------------------------------
void BlinkUSBStatus(void)
{
    static WORD led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(USBSuspendControl == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            if(mGetLED_1())
            {
                mLED_2_On();
            }
            else
            {
                mLED_2_Off();
            }
        }//end if
    }
    else
    {
        if(USBDeviceState == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(USBDeviceState == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(USBDeviceState == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(USBDeviceState == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(USBDeviceState == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(USBDeviceState == CONFIGURED_STATE)
        {
            if(led_count==0)
            {
                mLED_1_Toggle();
                if(mGetLED_1())
                {
                    mLED_2_Off();
                }
                else
                {
                    mLED_2_On();
                }
            }//end if
        }
    }

}//end BlinkUSBStatus




// ******************************************************************************************************
// ************** USB Callback Functions ****************************************************************
// ******************************************************************************************************
// The USB firmware stack will call the callback functions USBCBxxx() in response to certain USB related
// events.  For example, if the host PC is powering down, it will stop sending out Start of Frame (SOF)
// packets to your device.  In response to this, all USB devices are supposed to decrease their power
// consumption from the USB Vbus to <2.5mA each.  The USB module detects this condition (which according
// to the USB specifications is 3+ms of no bus activity/SOF packets) and then calls the USBCBSuspend()
// function.  You should modify these callback functions to take appropriate actions for each of these
// conditions.  For example, in the USBCBSuspend(), you may wish to add code that will decrease power
// consumption from Vbus to <2.5mA (such as by clock switching, turning off LEDs, putting the
// microcontroller to sleep, etc.).  Then, in the USBCBWakeFromSuspend() function, you may then wish to
// add code that undoes the power saving things done in the USBCBSuspend() function.

// The USBCBSendResume() function is special, in that the USB stack will not automatically call this
// function.  This function is meant to be called from the application firmware instead.  See the
// additional comments near the function.

// Note *: The "usb_20.pdf" specs indicate 500uA or 2.5mA, depending upon device classification. However,
// the USB-IF has officially issued an ECN (engineering change notice) changing this to 2.5mA for all 
// devices.  Make sure to re-download the latest specifications to get all of the newest ECNs.

/******************************************************************************
 * Function:        void USBCBSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Call back that is invoked when a USB suspend is detected
 *
 * Note:            None
 *****************************************************************************/
void USBCBSuspend(void)
{
	//Example power saving code.  Insert appropriate code here for the desired
	//application behavior.  If the microcontroller will be put to sleep, a
	//process similar to that shown below may be used:
	
	//ConfigureIOPinsForLowPower();
	//SaveStateOfAllInterruptEnableBits();
	//DisableAllInterruptEnableBits();
	//EnableOnlyTheInterruptsWhichWillBeUsedToWakeTheMicro();	//should enable at least USBActivityIF as a wake source
	//Sleep();
	//RestoreStateOfAllPreviouslySavedInterruptEnableBits();	//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.
	//RestoreIOPinsToNormal();									//Preferrably, this should be done in the USBCBWakeFromSuspend() function instead.

	//IMPORTANT NOTE: Do not clear the USBActivityIF (ACTVIF) bit here.  This bit is 
	//cleared inside the usb_device.c file.  Clearing USBActivityIF here will cause 
	//things to not work as intended.	
	

    #if defined(__C30__)
    #if 0
        U1EIR = 0xFFFF;
        U1IR = 0xFFFF;
        U1OTGIR = 0xFFFF;
        IFS5bits.USB1IF = 0;
        IEC5bits.USB1IE = 1;
        U1OTGIEbits.ACTVIE = 1;
        U1OTGIRbits.ACTVIF = 1;
        Sleep();
    #endif
    #endif
}


/******************************************************************************
 * Function:        void _USB1Interrupt(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the USB interrupt bit is set
 *					In this example the interrupt is only used when the device
 *					goes to sleep when it receives a USB suspend command
 *
 * Note:            None
 *****************************************************************************/
#if 0
void __attribute__ ((interrupt)) _USB1Interrupt(void)
{
    #if !defined(self_powered)
        if(U1OTGIRbits.ACTVIF)
        {       
            IEC5bits.USB1IE = 0;
            U1OTGIEbits.ACTVIE = 0;
            IFS5bits.USB1IF = 0;
        
            //USBClearInterruptFlag(USBActivityIFReg,USBActivityIFBitNum);
            USBClearInterruptFlag(USBIdleIFReg,USBIdleIFBitNum);
            //USBSuspendControl = 0;
        }
    #endif
}
#endif

/******************************************************************************
 * Function:        void USBCBWakeFromSuspend(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The host may put USB peripheral devices in low power
 *					suspend mode (by "sending" 3+ms of idle).  Once in suspend
 *					mode, the host may wake the device back up by sending non-
 *					idle state signalling.
 *					
 *					This call back is invoked when a wakeup from USB suspend 
 *					is detected.
 *
 * Note:            None
 *****************************************************************************/
void USBCBWakeFromSuspend(void)
{
	// If clock switching or other power savings measures were taken when
	// executing the USBCBSuspend() function, now would be a good time to
	// switch back to normal full power run mode conditions.  The host allows
	// a few milliseconds of wakeup time, after which the device must be 
	// fully back to normal, and capable of receiving and processing USB
	// packets.  In order to do this, the USB module must receive proper
	// clocking (IE: 48MHz clock must be available to SIE for full speed USB
	// operation).
}

/********************************************************************
 * Function:        void USBCB_SOF_Handler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB host sends out a SOF packet to full-speed
 *                  devices every 1 ms. This interrupt may be useful
 *                  for isochronous pipes. End designers should
 *                  implement callback routine as necessary.
 *
 * Note:            None
 *******************************************************************/
void USBCB_SOF_Handler(void)
{
    // No need to clear UIRbits.SOFIF to 0 here.
    // Callback caller is already doing that.
}

/*******************************************************************
 * Function:        void USBCBErrorHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The purpose of this callback is mainly for
 *                  debugging during development. Check UEIR to see
 *                  which error causes the interrupt.
 *
 * Note:            None
 *******************************************************************/
void USBCBErrorHandler(void)
{
    // No need to clear UEIR to 0 here.
    // Callback caller is already doing that.

	// Typically, user firmware does not need to do anything special
	// if a USB error occurs.  For example, if the host sends an OUT
	// packet to your device, but the packet gets corrupted (ex:
	// because of a bad connection, or the user unplugs the
	// USB cable during the transmission) this will typically set
	// one or more USB error interrupt flags.  Nothing specific
	// needs to be done however, since the SIE will automatically
	// send a "NAK" packet to the host.  In response to this, the
	// host will normally retry to send the packet again, and no
	// data loss occurs.  The system will typically recover
	// automatically, without the need for application firmware
	// intervention.
	
	// Nevertheless, this callback function is provided, such as
	// for debugging purposes.
}


/*******************************************************************
 * Function:        void USBCBCheckOtherReq(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        When SETUP packets arrive from the host, some
 * 					firmware must process the request and respond
 *					appropriately to fulfill the request.  Some of
 *					the SETUP packets will be for standard
 *					USB "chapter 9" (as in, fulfilling chapter 9 of
 *					the official USB specifications) requests, while
 *					others may be specific to the USB device class
 *					that is being implemented.  For example, a HID
 *					class device needs to be able to respond to
 *					"GET REPORT" type of requests.  This
 *					is not a standard USB chapter 9 request, and 
 *					therefore not handled by usb_device.c.  Instead
 *					this request should be handled by class specific 
 *					firmware, such as that contained in usb_function_hid.c.
 *
 * Note:            None
 *******************************************************************/
void USBCBCheckOtherReq(void)
{
    USBCheckHIDRequest();
}//end


/*******************************************************************
 * Function:        void USBCBStdSetDscHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USBCBStdSetDscHandler() callback function is
 *					called when a SETUP, bRequest: SET_DESCRIPTOR request
 *					arrives.  Typically SET_DESCRIPTOR requests are
 *					not used in most applications, and it is
 *					optional to support this type of request.
 *
 * Note:            None
 *******************************************************************/
void USBCBStdSetDscHandler(void)
{
    // Must claim session ownership if supporting this request
}//end


/*******************************************************************
 * Function:        void USBCBInitEP(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called when the device becomes
 *                  initialized, which occurs after the host sends a
 * 					SET_CONFIGURATION (wValue not = 0) request.  This 
 *					callback function should initialize the endpoints 
 *					for the device's usage according to the current 
 *					configuration.
 *
 * Note:            None
 *******************************************************************/
void USBCBInitEP(void)
{
    //enable the HID endpoint
    USBEnableEndpoint(HID_EP,USB_IN_ENABLED|USB_OUT_ENABLED|USB_HANDSHAKE_ENABLED|USB_DISALLOW_SETUP);

	lastOUTTransmission = HIDRxPacket(HID_EP,(BYTE*)&hid_report_out,1);

}

/********************************************************************
 * Function:        void USBCBSendResume(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        The USB specifications allow some types of USB
 * 					peripheral devices to wake up a host PC (such
 *					as if it is in a low power suspend to RAM state).
 *					This can be a very useful feature in some
 *					USB applications, such as an Infrared remote
 *					control	receiver.  If a user presses the "power"
 *					button on a remote control, it is nice that the
 *					IR receiver can detect this signalling, and then
 *					send a USB "command" to the PC to wake up.
 *					
 *					The USBCBSendResume() "callback" function is used
 *					to send this special USB signalling which wakes 
 *					up the PC.  This function may be called by
 *					application firmware to wake up the PC.  This
 *					function will only be able to wake up the host if
 *                  all of the below are true:
 *					
 *					1.  The USB driver used on the host PC supports
 *						the remote wakeup capability.
 *					2.  The USB configuration descriptor indicates
 *						the device is remote wakeup capable in the
 *						bmAttributes field.
 *					3.  The USB host PC is currently sleeping,
 *						and has previously sent your device a SET 
 *						FEATURE setup packet which "armed" the
 *						remote wakeup capability.   
 *
 *                  If the host has not armed the device to perform remote wakeup,
 *                  then this function will return without actually performing a
 *                  remote wakeup sequence.  This is the required behavior, 
 *                  as a USB device that has not been armed to perform remote 
 *                  wakeup must not drive remote wakeup signalling onto the bus;
 *                  doing so will cause USB compliance testing failure.
 *                  
 *					This callback should send a RESUME signal that
 *                  has the period of 1-15ms.
 *
 * Note:            This function does nothing and returns quickly, if the USB
 *                  bus and host are not in a suspended condition, or are 
 *                  otherwise not in a remote wakeup ready state.  Therefore, it
 *                  is safe to optionally call this function regularly, ex: 
 *                  anytime application stimulus occurs, as the function will
 *                  have no effect, until the bus really is in a state ready
 *                  to accept remote wakeup. 
 *
 *                  When this function executes, it may perform clock switching,
 *                  depending upon the application specific code in 
 *                  USBCBWakeFromSuspend().  This is needed, since the USB
 *                  bus will no longer be suspended by the time this function
 *                  returns.  Therefore, the USB module will need to be ready
 *                  to receive traffic from the host.
 *
 *                  The modifiable section in this routine may be changed
 *                  to meet the application needs. Current implementation
 *                  temporary blocks other functions from executing for a
 *                  period of ~3-15 ms depending on the core frequency.
 *
 *                  According to USB 2.0 specification section 7.1.7.7,
 *                  "The remote wakeup device must hold the resume signaling
 *                  for at least 1 ms but for no more than 15 ms."
 *                  The idea here is to use a delay counter loop, using a
 *                  common value that would work over a wide range of core
 *                  frequencies.
 *                  That value selected is 1800. See table below:
 *                  ==========================================================
 *                  Core Freq(MHz)      MIP         RESUME Signal Period (ms)
 *                  ==========================================================
 *                      48              12          1.05
 *                       4              1           12.6
 *                  ==========================================================
 *                  * These timing could be incorrect when using code
 *                    optimization or extended instruction mode,
 *                    or when having other interrupts enabled.
 *                    Make sure to verify using the MPLAB SIM's Stopwatch
 *                    and verify the actual signal on an oscilloscope.
 *******************************************************************/
void USBCBSendResume(void)
{
    static WORD delay_count;
    
    //First verify that the host has armed us to perform remote wakeup.
    //It does this by sending a SET_FEATURE request to enable remote wakeup,
    //usually just before the host goes to standby mode (note: it will only
    //send this SET_FEATURE request if the configuration descriptor declares
    //the device as remote wakeup capable, AND, if the feature is enabled
    //on the host (ex: on Windows based hosts, in the device manager 
    //properties page for the USB device, power management tab, the 
    //"Allow this device to bring the computer out of standby." checkbox 
    //should be checked).
    if(USBGetRemoteWakeupStatus() == TRUE) 
    {
        //Verify that the USB bus is in fact suspended, before we send
        //remote wakeup signalling.
        if(USBIsBusSuspended() == TRUE)
        {
            USBMaskInterrupts();
            
            //Clock switch to settings consistent with normal USB operation.
            USBCBWakeFromSuspend();
            USBSuspendControl = 0; 
            USBBusIsSuspended = FALSE;  //So we don't execute this code again, 
                                        //until a new suspend condition is detected.

            //Section 7.1.7.7 of the USB 2.0 specifications indicates a USB
            //device must continuously see 5ms+ of idle on the bus, before it sends
            //remote wakeup signalling.  One way to be certain that this parameter
            //gets met, is to add a 2ms+ blocking delay here (2ms plus at 
            //least 3ms from bus idle to USBIsBusSuspended() == TRUE, yeilds
            //5ms+ total delay since start of idle).
            delay_count = 3600U;        
            do
            {
                delay_count--;
            }while(delay_count);
            
            //Now drive the resume K-state signalling onto the USB bus.
            USBResumeControl = 1;       // Start RESUME signaling
            delay_count = 1800U;        // Set RESUME line for 1-13 ms
            do
            {
                delay_count--;
            }while(delay_count);
            USBResumeControl = 0;       //Finished driving resume signalling

            USBUnmaskInterrupts();
        }
    }
}


/*******************************************************************
 * Function:        BOOL USER_USB_CALLBACK_EVENT_HANDLER(
 *                        USB_EVENT event, void *pdata, WORD size)
 *
 * PreCondition:    None
 *
 * Input:           USB_EVENT event - the type of event
 *                  void *pdata - pointer to the event data
 *                  WORD size - size of the event data
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        This function is called from the USB stack to
 *                  notify a user application that a USB event
 *                  occured.  This callback is in interrupt context
 *                  when the USB_INTERRUPT option is selected.
 *
 * Note:            None
 *******************************************************************/
BOOL USER_USB_CALLBACK_EVENT_HANDLER(USB_EVENT event, void *pdata, WORD size)
{
    switch( (INT)event )
    {
        case EVENT_TRANSFER:
            //Add application specific callback task or callback function here if desired.
            break;
        case EVENT_SOF:
            USBCB_SOF_Handler();
            break;
        case EVENT_SUSPEND:
            USBCBSuspend();
            break;
        case EVENT_RESUME:
            USBCBWakeFromSuspend();
            break;
        case EVENT_CONFIGURED: 
            USBCBInitEP();
            break;
        case EVENT_SET_DESCRIPTOR:
            USBCBStdSetDscHandler();
            break;
        case EVENT_EP0_REQUEST:
            USBCBCheckOtherReq();
            break;
        case EVENT_BUS_ERROR:
            USBCBErrorHandler();
            break;
        case EVENT_TRANSFER_TERMINATED:
            //Add application specific callback task or callback function here if desired.
            //The EVENT_TRANSFER_TERMINATED event occurs when the host performs a CLEAR
            //FEATURE (endpoint halt) request on an application endpoint which was 
            //previously armed (UOWN was = 1).  Here would be a good place to:
            //1.  Determine which endpoint the transaction that just got terminated was 
            //      on, by checking the handle value in the *pdata.
            //2.  Re-arm the endpoint if desired (typically would be the case for OUT 
            //      endpoints).
            break;
        default:
            break;
    }      
    return TRUE; 
}


// *****************************************************************************
// ************** USB Class Specific Callback Function(s) **********************
// *****************************************************************************

/********************************************************************
 * Function:        void USBHIDCBSetReportHandler(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        USBHIDCBSetReportHandler() is used to respond to
 *					the HID device class specific SET_REPORT control
 *					transfer request (starts with SETUP packet on EP0 OUT).  
 * Note:            
 *******************************************************************/
void USBHIDCBSetReportHandler(void)
{
	//Prepare to receive the keyboard LED state data through a SET_REPORT
	//control transfer on endpoint 0.  The host should only send 1 byte,
	//since this is all that the report descriptor allows it to send.
	USBEP0Receive((BYTE*)&CtrlTrfData, USB_EP0_BUFF_SIZE, USBHIDCBSetReportComplete);
}

//Secondary callback function that gets called when the above
//control transfer completes for the USBHIDCBSetReportHandler()
void USBHIDCBSetReportComplete(void)
{
	//1 byte of LED state data should now be in the CtrlTrfData buffer.

	//Num Lock LED state is in Bit0.
	if(CtrlTrfData[0] & 0x01)	//Make LED1 and LED2 match Num Lock state.
	{
		mLED_1_On();
		mLED_2_On();
	}
	else
	{
		mLED_1_Off();
		mLED_2_Off();			
	}
	
	//Stop toggling the LEDs, so you can temporily see the Num lock LED state instead.
	//Once the CountdownTimerToShowUSBStatusOnLEDs reaches 0, the LEDs will go back to showing USB state instead.
	BlinkStatusValid = FALSE;	
	CountdownTimerToShowUSBStatusOnLEDs = 140000; 
}	
/** EOF Keyboard.c **********************************************/
#endif
